from django.urls import path
from jnt_admin_tools.dashboard import views

urlpatterns = [
    path(
        "set_preferences/<dashboard_id>/",
        views.set_preferences,
        name="admin-tools-dashboard-set-preferences",
    ),
]



from .autocomplete_fields import AdminAutocompleteFieldsMixin
from .generic_foreign_key import GenericForeignKeyAdminMixin
from .generic_foreign_key_inline import GenericForeignKeyInlineAdminMixin
from .admin_clickable_links import AdminClickableLinksMixin
from .autocomplete_changelist_filters import (
    AdminAutocompleteChangelistFiltersMixin,
)



"""
This module contains some utils for easy deprecation warnings.
"""
import warnings


def import_path_is_changed(old_name, new_name):
    class ImportDeprecationMixin:
        def __new__(cls, *args, **kwargs):
            klass = super(ImportDeprecationMixin, cls).__new__(cls)
            msg = "%s: %s is deprecated. Please use %s instead." % (
                klass,
                old_name,
                new_name,
            )
            warnings.warn(msg, DeprecationWarning)
            return klass

    return ImportDeprecationMixin



from typing import Optional

from django.contrib.admin.utils import lookup_field
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
from django.db import models

from jnt_admin_tools.services.object_links import (
    get_display_for_gfk,
    get_display_for_many,
    object_change_link,
)


def get_present_admin_readonly_field(  # noqa: WPS212
    admin_readonly_field,
) -> Optional[str]:
    """Get present for AdminReadonlyField."""
    field_name, instance, model_admin = (
        admin_readonly_field.field["field"],
        admin_readonly_field.form.instance,
        admin_readonly_field.model_admin,
    )

    if not instance or admin_readonly_field.is_checkbox:
        return None

    try:
        field, attr, field_value = lookup_field(
            field_name, instance, model_admin
        )
        if field is None:
            field = instance._meta.get_field(field_name)
    except (AttributeError, ValueError, ObjectDoesNotExist, FieldDoesNotExist):
        return None

    present = _get_from_readonly_widget(
        model_admin,
        field_name,
        field_value,
    )

    if present:
        return present

    if isinstance(field_value, str):
        if getattr(field, "choices", None):
            field_value = _get_choice_present(field, field_value)
        return field_value

    if field.is_relation:
        return _get_display_relation(field, field_value)

    return None


def _get_from_readonly_widget(
    model_admin, field_name, field_value
) -> Optional[str]:
    if not model_admin:
        return None

    formfield = model_admin.form.base_fields.get(field_name)

    if not formfield:
        return None

    readonly_widgets = getattr(
        model_admin.form.Meta,
        "readonly_widgets",
        {},
    )
    readonly_widget = readonly_widgets.get(field_name)

    if not readonly_widget:
        readonly_widget = getattr(formfield, "readonly_widget", None)

    if readonly_widget:
        return readonly_widget(formfield=formfield).render(
            field_name, field_value
        )

    return None


def _get_display_relation(field, instance) -> Optional[str]:
    if not instance:
        return None

    if isinstance(field, GenericForeignKey):
        return get_display_for_gfk(instance)
    elif field.many_to_many or field.one_to_many:
        return get_display_for_many(instance.all())

    return object_change_link(instance)


def _get_choice_present(field: models.CharField, field_value: str) -> str:
    """Get present for choice."""
    choice = [
        present
        for choice_value, present in field.choices
        if choice_value == field_value
    ]

    if choice:
        field_value = choice[0]

    return field_value



from itertools import chain

from django.contrib.admin.checks import BaseModelAdminChecks
from django.contrib.admin.utils import flatten_fieldsets
from django.db.models import ForeignKey, ManyToManyField


class AdminAutocompleteFieldsMixin:
    def get_autocomplete_fields(self, request):
        autocomplete_fields = super().get_autocomplete_fields(request)

        if autocomplete_fields:
            return autocomplete_fields

        admin_fields = self._get_admin_fields(request)
        relation_fields = self._get_relation_fields()

        if not admin_fields:
            return tuple(relation_fields)

        return tuple(set(admin_fields) & set(relation_fields))

    def _get_relation_fields(self):
        return (
            field.name
            for field in self.model._meta.get_fields()
            if self._is_relation_field(field)
        )

    def _get_admin_fields(self, request):
        if self.fields:
            return self.get_fields(request)
        elif self.fieldsets:
            return flatten_fieldsets(self.get_fieldsets(request))

    def _is_relation_field(self, field):
        return isinstance(field, (ForeignKey, ManyToManyField)) and bool(
            self.admin_site._registry.get(field.remote_field.model)
        )

    def check(self, **kwargs):
        return [
            *super().check(**kwargs),
            *self._check_autocomplete_mixin(),
        ]

    def _check_autocomplete_mixin(self):
        if "autocomplete_fields" in self.__class__.__dict__.keys():
            return []

        check_autocomplete_fields_item = (
            BaseModelAdminChecks()._check_autocomplete_fields_item
        )

        return list(
            chain.from_iterable(
                [
                    check_autocomplete_fields_item(
                        self, field_name, f"autocomplete_fields[{index:d}]"
                    )
                    for index, field_name in enumerate(
                        self.get_autocomplete_fields(None)
                    )
                ]
            )
        )



"""
This module contains the base classes for the dashboard and dashboard modules.
"""
from django.conf import settings
from django.db import models

# warnings for deprecated imports
from jnt_admin_tools.deprecate_utils import import_path_is_changed
from jnt_admin_tools.dashboard import dashboards
from jnt_admin_tools.dashboard import modules

user_model = getattr(settings, "AUTH_USER_MODEL", "auth.User")


class DashboardPreferences(models.Model):
    """
    This model represents the dashboard preferences for a user.
    """

    user = models.ForeignKey(user_model, on_delete=models.CASCADE)
    data = models.TextField()
    dashboard_id = models.CharField(max_length=100)

    def __str__(self):
        return "{0} dashboard preferences".format(self.user.get_username())

    class Meta:
        db_table = "admin_tools_dashboard_preferences"
        unique_together = (
            "user",
            "dashboard_id",
        )
        ordering = ("user",)


class Dashboard(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.Dashboard",
        "jnt_admin_tools.dashboard.Dashboard",
    ),
    dashboards.Dashboard,
):
    pass


class DefaultIndexDashboard(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.DefaultIndexDashboard",
        "jnt_admin_tools.dashboard.DefaultIndexDashboard",
    ),
    dashboards.DefaultIndexDashboard,
):
    pass


class DefaultAppIndexDashboard(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.DefaultAppIndexDashboard",
        "jnt_admin_tools.dashboard.DefaultAppIndexDashboard",
    ),
    dashboards.DefaultAppIndexDashboard,
):
    pass


class AppIndexDashboard(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.AppIndexDashboard",
        "jnt_admin_tools.dashboard.AppIndexDashboard",
    ),
    dashboards.AppIndexDashboard,
):
    pass


class DashboardModule(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.DashboardModule",
        "jnt_admin_tools.dashboard.modules.DashboardModule",
    ),
    modules.DashboardModule,
):
    pass


class AppListDashboardModule(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.AppListDashboardModule",
        "jnt_admin_tools.dashboard.modules.AppList",
    ),
    modules.AppList,
):
    pass


class ModelListDashboardModule(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.ModelListDashboardModule",
        "jnt_admin_tools.dashboard.modules.ModelList",
    ),
    modules.ModelList,
):
    pass


class LinkListDashboardModule(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.LinkListDashboardModule",
        "jnt_admin_tools.dashboard.modules.LinkList",
    ),
    modules.LinkList,
):
    pass


class FeedDashboardModule(
    import_path_is_changed(
        "jnt_admin_tools.dashboard.models.FeedDashboardModule",
        "jnt_admin_tools.dashboard.modules.Feed",
    ),
    modules.Feed,
):
    pass
